package models

import (
	"eta/eta_index_lib/global"
	"eta/eta_index_lib/utils"
	"fmt"
	"gorm.io/gorm"
	"strconv"
	"strings"
	"time"
)

type BaseFromClarksonsIndex struct {
	BaseFromClarksonsIndexId int `gorm:"primaryKey"`
	//BaseFromClarksonsIndexId int       `orm:"pk"`
	ClassifyId int    `description:"指标分类id"`
	IndexCode  string `description:"指标编码"`
	IndexName  string `description:"指标名称"`
	Unit       string `description:"单位"`
	Frequency  string `description:"频度"`
	Sort       int    `description:"排序"`
	CreateTime time.Time
	ModifyTime time.Time
}

type BaseFromClarksonsData struct {
	BaseFromClarksonsDataId  int       `gorm:"column:base_from_clarksons_data_id;primaryKey"`
	BaseFromClarksonsIndexId int       `description:"指标id"`
	IndexCode                string    `description:"指标编码"`
	DataTime                 string    `description:"数据日期"`
	Value                    string    `description:"数据值"`
	CreateTime               time.Time `description:"创建时间"`
	ModifyTime               time.Time `description:"修改时间"`
}

func (e *BaseFromClarksonsData) AfterFind(db *gorm.DB) (err error) {
	e.DataTime = utils.GormDateStrToDateStr(e.DataTime)

	return
}

// 查询指标
func GetBaseFromClarksonsIndex() (items []*BaseFromClarksonsIndex, err error) {
	//o := orm.NewOrm()
	sql := `SELECT * FROM base_from_clarksons_index`
	//_, err = o.Raw(sql).QueryRows(&items)
	err = global.DEFAULT_DB.Raw(sql).Find(&items).Error
	return
}

// 查询指标
func GetBaseFromClarksonsData() (items []*BaseFromClarksonsData, err error) {
	//o := orm.NewOrm()
	sql := `SELECT * FROM base_from_clarksons_data`
	//_, err = o.Raw(sql).QueryRows(&items)
	err = global.DEFAULT_DB.Raw(sql).Find(&items).Error
	return
}

// 添加数据
func AddBaseFromClarksonsIndex(item *BaseFromClarksonsIndex) (lastId int64, err error) {
	//o := orm.NewOrm()
	//lastId, err = o.Insert(item)
	err = global.DEFAULT_DB.Create(&item).Error
	if err != nil {
		return
	}
	lastId = int64(item.BaseFromClarksonsIndexId)
	return
}

func AddBaseFromClarksonsDataMuti(items []*BaseFromClarksonsData) (count int64, err error) {
	dbResult := global.DEFAULT_DB.CreateInBatches(&items, utils.MultiAddNum)
	err = dbResult.Error
	count = dbResult.RowsAffected
	return
}

func UpdateBaseFromClarksonsIndex(item *BaseFromClarksonsIndex) (err error) {
	//o := orm.NewOrm()
	sql := `UPDATE base_from_clarksons_index SET frequency=?  WHERE index_code=? `
	//_, err = o.Raw(sql, item.Frequency, item.IndexCode).Exec()
	err = global.DEFAULT_DB.Exec(sql, item.Frequency, item.IndexCode).Error
	return
}

func AddEdbDataFromClarksons(edbCode string) (err error) {
	//o := orm.NewOrm()
	coalBaseDataAll, err := GetClarksonsindexByCode(edbCode)
	if err != nil && !utils.IsErrNoRow(err) {
		return
	}
	//if err != nil && !utils.IsErrNoRow(err) {
	//	return
	//}
	var isAdd bool
	addSql := ` INSERT INTO edb_data_clarksons(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	existMap := make(map[string]string)
	for _, sv := range coalBaseDataAll {
		eDate := sv.DataTime
		var timeStr string
		var dataTime time.Time
		var sDataTime string
		var timestamp int64

		sDataTime = eDate
		dataTime, err = time.ParseInLocation(utils.FormatDate, eDate, time.Local)
		if err != nil {
			fmt.Println("time.Parse Err:" + eDate)
			return err
		}
		timestamp = dataTime.UnixNano() / 1e6
		timeStr = fmt.Sprintf("%d", timestamp)

		value := strings.Replace(sv.Value, "%", "", -1)
		if _, ok := existMap[sDataTime]; !ok {
			addSql += GetAddSql("0", edbCode, sDataTime, timeStr, value)
			fmt.Println("edbCode：", edbCode)
			fmt.Println("sDataTime：", sDataTime)
			fmt.Println("timeStr：", timeStr)
			fmt.Println("value：", value)
			isAdd = true
		}
		existMap[eDate] = value
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		utils.FileLog.Info("addSql:" + addSql)
		//_, err = o.Raw(addSql).Exec()
		err = global.DEFAULT_DB.Exec(addSql).Error
		if err != nil {
			return err
		}
	}
	return
}

func RefreshEdbDataFromClarksons(edbInfoId int, edbCode, startDate string) (err error) {
	source := utils.DATA_SOURCE_CLARKSONS
	subSource := utils.DATA_SUB_SOURCE_EDB
	//o := orm.NewOrm()
	if err != nil {
		return
	}

	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据
	var condition string
	var pars []interface{}

	if edbCode != "" {
		condition += " AND index_code=? "
		pars = append(pars, edbCode)
	}

	if startDate != "" {
		condition += " AND data_time>=? "
		pars = append(pars, startDate)
	}

	glDataList, err := GetClarksonsDataByTradeCode(condition, pars)
	if err != nil {
		return
	}

	// 真实数据的最大日期  ， 插入规则配置的日期
	var realDataMaxDate, edbDataInsertConfigDate time.Time
	var edbDataInsertConfig *EdbDataInsertConfig
	var isFindConfigDateRealData bool //是否找到配置日期的实际数据的值
	{
		edbDataInsertConfig, err = GetEdbDataInsertConfigByEdbId(edbInfoId)
		if err != nil && !utils.IsErrNoRow(err) {
			return
		}
		if edbDataInsertConfig != nil {
			edbDataInsertConfigDate = edbDataInsertConfig.Date
		}
	}

	//获取指标所有数据
	var existCondition string
	var existPars []interface{}

	existCondition += " AND edb_info_id=? "
	existPars = append(existPars, edbInfoId)
	if startDate != "" {
		existCondition += " AND data_time>=? "
		existPars = append(existPars, startDate)
	}

	existList, err := GetEdbDataByCondition(source, subSource, existCondition, existPars)
	if err != nil {
		return err
	}
	existMap := make(map[string]*EdbInfoSearchData)
	for _, v := range existList {
		existMap[v.DataTime] = v
	}

	addSql := ` INSERT INTO edb_data_clarksons(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	var isAdd bool
	for _, v := range glDataList {
		var value string
		value = strings.Replace(v.Value, "%", "", -1)
		item := v
		itemValue := value
		if _, ok := existMap[v.DataTime]; !ok {
			eDate := item.DataTime
			var timeStr string
			var dataTime time.Time
			var sDataTime string
			var timestamp int64

			sDataTime = eDate
			dataTime, err = time.ParseInLocation(utils.FormatDate, eDate, time.Local)
			if err != nil {
				fmt.Println("time.Parse Err:" + eDate)
				return err
			}
			timestamp = dataTime.UnixNano() / 1e6
			timeStr = fmt.Sprintf("%d", timestamp)

			sValue := itemValue
			if sValue != "" {
				saveValue := sValue

				if findItem, ok := existMap[eDate]; !ok {
					addSql += GetAddSql(edbInfoIdStr, edbCode, sDataTime, timeStr, saveValue)
					isAdd = true
				} else {
					if findItem != nil && utils.SubFloatToString(findItem.Value, 30) != sValue {
						err = ModifyEdbDataById(source, subSource, findItem.EdbDataId, sValue)
						if err != nil {
							return err
						}
					}
				}
			}

			// 下面代码主要目的是处理掉手动插入的数据判断
			{
				if realDataMaxDate.IsZero() || dataTime.After(realDataMaxDate) {
					realDataMaxDate = dataTime
				}
				if edbDataInsertConfigDate.IsZero() || dataTime.Equal(edbDataInsertConfigDate) {
					isFindConfigDateRealData = true
				}
			}

		}
	}

	// 处理手工数据补充的配置
	HandleConfigInsertEdbData(realDataMaxDate, edbDataInsertConfig, edbInfoId, source, subSource, existMap, isFindConfigDateRealData)

	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		//_, err = o.Raw(addSql).Exec()
		err = global.DEFAULT_DB.Exec(addSql).Error
		if err != nil {
			return err
		}
	}

	return
}

// GetClarksonsindexByCode
func GetClarksonsindexByCode(indexCode string) (items []*BaseFromClarksonsData, err error) {
	//o := orm.NewOrm()
	sql := "SELECT * FROM base_from_clarksons_data WHERE index_code=? "
	//_, err = o.Raw(sql, indexCode).QueryRows(&items)
	err = global.DEFAULT_DB.Raw(sql, indexCode).Find(&items).Error
	return
}

func GetClarksonsDataByTradeCode(condition string, pars []interface{}) (item []*BaseFromClarksonsData, err error) {
	sql := ` SELECT * FROM base_from_clarksons_data WHERE 1=1 `
	//o := orm.NewOrm()
	if condition != "" {
		sql += condition
	}
	sql += ` ORDER BY data_time DESC `
	//_, err = o.Raw(sql, pars).QueryRows(&item)
	err = global.DEFAULT_DB.Raw(sql, pars...).Find(&item).Error
	return
}
