package models

import (
	"eta/eta_index_lib/global"
	"eta/eta_index_lib/utils"
	"fmt"
	"gorm.io/gorm"
	"strconv"
	"strings"
	"time"
)

// BaseFromFenwei 涌溢咨询
type BaseFromFenwei struct{}

type BaseFromFenweiData struct {
	FenweiDataId int `gorm:"column:fenwei_data_id;primaryKey"`
	//enweiDataId  int `orm:"column(fenwei_data_id);pk"`
	FenweiIndexId int
	IndexCode     string
	DataTime      string
	Value         string
	CreateTime    time.Time
	ModifyTime    time.Time
	DataTimestamp int64
}

func (m *BaseFromFenweiData) TableName() string {
	return "base_from_fenwei_data"
}

func (e *BaseFromFenweiData) AfterFind(db *gorm.DB) (err error) {
	e.DataTime = utils.GormDateStrToDateStr(e.DataTime)

	return
}

func GetBaseFromFenweiDataByCondition(condition string, pars []interface{}) (list []*BaseFromFenweiData, err error) {
	//o := orm.NewOrm()
	sql := `SELECT * FROM base_from_fenwei_data WHERE 1=1 `
	if condition != "" {
		sql += condition
	}
	//_, err = o.Raw(sql, pars).QueryRows(&list)
	err = global.DEFAULT_DB.Raw(sql, pars...).Find(&list).Error
	return
}

// Add 添加
func (obj BaseFromFenwei) Add(edbCode string) (err error) {
	//o := orm.NewOrm()

	var condition string
	var pars []interface{}
	if edbCode != "" {
		condition += " AND index_code=? "
		pars = append(pars, edbCode)
	}
	fenweiBaseDataAll, err := GetBaseFromFenweiDataByCondition(condition, pars)
	if err != nil && !utils.IsErrNoRow(err) {
		return
	}
	var isAdd bool
	addSql := ` INSERT INTO edb_data_fenwei(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	addSql = utils.ReplaceDriverKeywords("", addSql)
	existMap := make(map[string]string)
	for _, sv := range fenweiBaseDataAll {
		eDate := sv.DataTime
		dataTime, err := time.Parse(utils.FormatDate, eDate)
		if err != nil {
			return err
		}
		timestamp := dataTime.UnixNano() / 1e6
		timeStr := fmt.Sprintf("%d", timestamp)
		if _, ok := existMap[eDate]; !ok {
			addSql += GetAddSql("0", edbCode, eDate, timeStr, sv.Value)
			isAdd = true
		}
		existMap[eDate] = sv.Value
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		utils.FileLog.Info("addSql:" + addSql)
		//_, err = o.Raw(addSql).Exec()
		err = global.DEFAULT_DB.Exec(addSql).Error
		if err != nil {
			return err
		}
	}
	return
}

// Refresh 刷新汾渭数据指标数据
func (obj BaseFromFenwei) Refresh(edbInfoId int, edbCode, startDate string) (err error) {
	source := obj.GetSource()
	//o := orm.NewOrm()
	if err != nil {
		return
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据
	var condition string
	var pars []interface{}

	if edbCode != "" {
		condition += " AND index_code=? "
		pars = append(pars, edbCode)
	}

	if startDate != "" {
		condition += " AND data_time>=? "
		pars = append(pars, startDate)
	}

	fenweiDataList, err := GetBaseFromFenweiDataByCondition(condition, pars)
	if err != nil {
		return
	}

	// 真实数据的最大日期  ， 插入规则配置的日期
	var realDataMaxDate, edbDataInsertConfigDate time.Time
	var edbDataInsertConfig *EdbDataInsertConfig
	var isFindConfigDateRealData bool //是否找到配置日期的实际数据的值
	{
		edbDataInsertConfig, err = GetEdbDataInsertConfigByEdbId(edbInfoId)
		if err != nil && !utils.IsErrNoRow(err) {
			return
		}
		if edbDataInsertConfig != nil {
			edbDataInsertConfigDate = edbDataInsertConfig.Date
		}
	}

	var existCondition string
	var existPars []interface{}

	existCondition += " AND edb_info_id=? "
	existPars = append(existPars, edbInfoId)
	if startDate != "" {
		existCondition += " AND data_time>=? "
		existPars = append(existPars, startDate)
	}

	existList, err := GetEdbDataByCondition(source, 0, existCondition, existPars)
	if err != nil {
		return err
	}
	existMap := make(map[string]*EdbInfoSearchData)
	for _, v := range existList {
		existMap[v.DataTime] = v
	}
	addSql := ` INSERT INTO edb_data_fenwei(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	addSql = utils.ReplaceDriverKeywords("", addSql)
	var isAdd bool
	for _, v := range fenweiDataList {
		item := v
		eDate := item.DataTime
		dataTime, err := time.ParseInLocation(utils.FormatDate, eDate, time.Local)
		if err != nil {
			return err
		}
		if findItem, ok := existMap[v.DataTime]; !ok {
			sValue := item.Value

			timestamp := dataTime.UnixNano() / 1e6
			timeStr := fmt.Sprintf("%d", timestamp)

			addSql += GetAddSql(edbInfoIdStr, edbCode, eDate, timeStr, sValue)
			isAdd = true
		} else {
			if findItem != nil && utils.SubFloatToString(findItem.Value, 30) != item.Value {
				err = ModifyEdbDataById(source, 0, findItem.EdbDataId, item.Value)
				if err != nil {
					return err
				}
			}
		}

		// 下面代码主要目的是处理掉手动插入的数据判断
		{
			if realDataMaxDate.IsZero() || dataTime.After(realDataMaxDate) {
				realDataMaxDate = dataTime
			}
			if edbDataInsertConfigDate.IsZero() || dataTime.Equal(edbDataInsertConfigDate) {
				isFindConfigDateRealData = true
			}
		}
	}

	// 处理手工数据补充的配置
	HandleConfigInsertEdbData(realDataMaxDate, edbDataInsertConfig, edbInfoId, source, 0, existMap, isFindConfigDateRealData)

	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		//_, err = o.Raw(addSql).Exec()
		err = global.DEFAULT_DB.Exec(addSql).Error
		if err != nil {
			fmt.Println("RefreshEdbDataFromBaiinfo add Err", err.Error())
			return
		}
	}
	return
}

// GetSource 获取来源编码id
func (obj BaseFromFenwei) GetSource() int {
	return utils.DATA_SOURCE_FENWEI
}

// GetSourceName 获取来源名称
func (obj BaseFromFenwei) GetSourceName() string {
	return utils.DATA_SOURCE_NAME_FENWEI
}

type BaseFromFenweiIndex struct {
	FenweiIndexId int64 `gorm:"column:fenwei_index_id;primaryKey"`
	//FenweiIndexId int64 `orm:"column(fenwei_index_id);pk"`
	IndexCode    string
	IndexName    string
	Frequency    string
	Unit         string
	ClassifyId   int
	StartDate    string
	EndDate      string
	Sort         int
	TerminalCode string
	CreateTime   time.Time
	ModifyTime   time.Time
}

func (e *BaseFromFenweiIndex) AfterFind(db *gorm.DB) (err error) {
	e.StartDate = utils.GormDateStrToDateStr(e.StartDate)
	e.EndDate = utils.GormDateStrToDateStr(e.EndDate)

	return
}

func (m *BaseFromFenweiIndex) TableName() string {
	return "base_from_fenwei_index"
}

func (m *BaseFromFenweiIndex) Add() (lastId int64, err error) {
	//o := orm.NewOrm()
	//lastId, err = o.Insert(m)
	err = global.DEFAULT_DB.Create(&m).Error
	if err != nil {
		return
	}
	lastId = int64(m.FenweiIndexId)
	return
}

func (m *BaseFromFenweiIndex) Update(updateCols []string) (err error) {
	//o := orm.NewOrm()
	//_, err = o.Update(m, updateCols...)
	err = global.DEFAULT_DB.Model(&m).Select(updateCols).Updates(&m).Error

	return
}

func (m *BaseFromFenweiIndex) ModifyIndexMaxAndMinDate(indexCode string, item *EdbInfoMaxAndMinInfo) (err error) {
	//o := orm.NewOrm()
	sql := ` UPDATE base_from_fenwei_index SET start_date=?,end_date=?,latest_value=?,modify_time=NOW() WHERE index_code=? `
	//_, err = o.Raw(sql, item.MinDate, item.MaxDate, item.LatestValue, indexCode).Exec()
	err = global.DEFAULT_DB.Exec(sql, item.MinDate, item.MaxDate, item.LatestValue, indexCode).Error
	return
}

func (m *BaseFromFenweiIndex) GetByIndexCode(indexCode string) (item *BaseFromFenweiIndex, err error) {
	//o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_fenwei_index WHERE index_code=? `
	//err = o.Raw(sql, indexCode).QueryRow(&item)
	err = global.DEFAULT_DB.Raw(sql, indexCode).First(&item).Error
	return
}

func (m *BaseFromFenweiIndex) GetItemsByCondition(condition string, pars []interface{}, fieldArr []string, orderRule string) (items []*BaseFromFenweiIndex, err error) {
	//o := orm.NewOrm()
	fields := strings.Join(fieldArr, ",")
	if len(fieldArr) == 0 {
		fields = `*`
	}
	order := `ORDER BY create_time DESC`
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT %s FROM base_from_fenwei_index WHERE 1=1 %s %s`, fields, condition, order)
	//_, err = o.Raw(sql, pars).QueryRows(&items)
	err = global.DEFAULT_DB.Raw(sql, pars...).Find(&items).Error
	return
}

type BaseFromFenweiIndexList struct {
	//FenweiIndexId int64 `orm:"column(fenwei_index_id);pk"`
	FenweiIndexId int64 `gorm:"column:fenwei_index_id;primaryKey"`
	IndexCode     string
	IndexName     string
	Frequency     string
	Unit          string
	ClassifyId    int
	StartDate     string
	EndDate       string
	TerminalCode  string
	CreateTime    string
	ModifyTime    string
}

func (y *BaseFromFenweiData) GetByIndexCode(indexCode string) (list []*BaseFromFenweiData, err error) {
	//o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_fenwei_data WHERE index_code=? `
	//_, err = o.Raw(sql, indexCode).QueryRows(&list)
	err = global.DEFAULT_DB.Raw(sql, indexCode).Find(&list).Error
	return
}

func (y *BaseFromFenweiData) AddMulti(item []*BaseFromFenweiData) (err error) {
	//o := orm.NewOrm()
	//_, err = o.InsertMulti(1, item)
	err = global.DEFAULT_DB.CreateInBatches(&item, 1).Error
	return
}

// HandleFenweiExcelData 汾渭数据的excel数据
type HandleFenweiExcelData struct {
	//ClassifyName string `description:"指标目录"`
	//ClassifySort int    `description:"指标目录排序号"`
	IndexName    string `description:"指标名称"`
	IndexCode    string `description:"指标编码"`
	Unit         string `description:"单位"`
	Sort         int    `description:"排序号"`
	Frequency    string `description:"频度"`
	ClassifyId   int    `description:"分类ID"`
	ExcelDataMap map[string]string
}

type HandleFenweiExcelDataReq struct {
	List         []*HandleFenweiExcelData
	TerminalCode string `description:"编码"`
}

type FenWeiNetIndexInfo struct {
	FenweiIndexId int64  `gorm:"column:fenwei_index_id;primaryKey"`
	IndexName     string `description:"指标名称"`
	//IndexCode    string  `description:"指标编码"`
	Unit         string      `description:"单位"`
	Frequency    string      `description:"频度"`
	TerminalCode string      `description:"编码"`
	ClassifyName string      `description:"分类名称"`
	DataTime     string      `description:"数据时间"`
	Value        interface{} `description:"数据值"`
}

type HandleFenWeiNetDataReq struct {
	List         []*FenWeiNetIndexInfo
	TerminalCode string `description:"编码"`
}

func (y *BaseFromFenweiData) GetMaxAndMinDateByIndexCode(indexCode string) (item *EdbInfoMaxAndMinInfo, err error) {
	//o := orm.NewOrm()
	sql := ` SELECT MIN(data_time) AS min_date,MAX(data_time) AS max_date,MIN(value) AS min_value,MAX(value) AS max_value FROM base_from_fenwei_data WHERE index_code=? `
	//err = o.Raw(sql, indexCode).QueryRow(&item)
	err = global.DEFAULT_DB.Raw(sql, indexCode).First(&item).Error
	// 获取最新值
	var lastVal float64
	sql = ` SELECT value AS latest_value FROM base_from_fenwei_data WHERE index_code = ? ORDER BY data_time DESC LIMIT 1 `
	//err = o.Raw(sql, indexCode).QueryRow(&lastVal)
	err = global.DEFAULT_DB.Raw(sql, indexCode).Scan(&lastVal).Error
	if err != nil {
		return
	}
	item.LatestValue = lastVal
	return
}

// BaseFromFenweiClassify 汾渭数据分类表
type BaseFromFenweiClassify struct {
	//ClassifyId      int       `orm:"column(classify_id);pk"`
	ClassifyId      int       `gorm:"column:classify_id;primaryKey"`
	ClassifyName    string    `description:"分类名称"`
	ParentId        int       `description:"父级id"`
	SysUserId       int       `description:"创建人id"`
	SysUserRealName string    `description:"创建人姓名"`
	Level           int       `description:"层级"`
	Sort            int       `description:"排序字段，越小越靠前，默认值：10"`
	ModifyTime      time.Time `description:"修改时间"`
	CreateTime      time.Time `description:"创建时间"`
}

func (y *BaseFromFenweiClassify) Add() (lastId int64, err error) {
	//o := orm.NewOrm()
	//lastId, err = o.Insert(y)
	err = global.DEFAULT_DB.Create(&y).Error
	if err != nil {
		return
	}
	lastId = int64(y.ClassifyId)
	return
}

func (y *BaseFromFenweiClassify) Update(updateCols []string) (err error) {
	//o := orm.NewOrm()
	//_, err = o.Update(y, updateCols...)
	err = global.DEFAULT_DB.Model(&y).Select(updateCols).Updates(&y).Error
	return
}

func (y *BaseFromFenweiClassify) GetByClassifyName(classifyName string) (item []*BaseFromFenweiClassify, err error) {
	//o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_fenwei_classify WHERE classify_name=? `
	//_, err = o.Raw(sql, classifyName).QueryRows(&item)

	err = global.DEFAULT_DB.Raw(sql, classifyName).Find(&item).Error
	if utils.IsErrNoRow(err) {
		return nil, nil
	}
	return
}

func (y *BaseFromFenweiClassify) GetItemsByCondition(condition string, pars []interface{}, fieldArr []string, orderRule string) (items []*BaseFromFenweiClassify, err error) {
	//o := orm.NewOrm()
	fields := strings.Join(fieldArr, ",")
	if len(fieldArr) == 0 {
		fields = `*`
	}
	order := `ORDER BY create_time DESC`
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT %s FROM base_from_fenwei_classify WHERE 1=1 %s %s`, fields, condition, order)
	//_, err = o.Raw(sql, pars).QueryRows(&items)
	err = global.DEFAULT_DB.Raw(sql, pars...).Find(&items).Error
	return
}

// BaseFromFenweiClassifyItem 汾渭数据分类信息
type BaseFromFenweiClassifyItem struct {
	ClassifyId   int                           `description:"分类ID"`
	ClassifyName string                        `description:"分类名称"`
	ParentId     int                           `description:"父级id"`
	Level        int                           `description:"层级"`
	Sort         int                           `description:"排序字段"`
	CreateTime   string                        `description:"创建时间"`
	ModifyTime   string                        `description:"修改时间"`
	Child        []*BaseFromFenweiClassifyItem `description:"子分类"`
}

func (y *BaseFromFenweiClassify) Format2Item(origin *BaseFromFenweiClassify) (item *BaseFromFenweiClassifyItem) {
	if origin == nil {
		return
	}
	item = new(BaseFromFenweiClassifyItem)
	item.ClassifyId = origin.ClassifyId
	item.ClassifyName = origin.ClassifyName
	item.ParentId = origin.ParentId
	item.Level = origin.Level
	item.Sort = origin.Sort
	item.CreateTime = utils.TimeTransferString(utils.FormatDateTime, origin.CreateTime)
	item.ModifyTime = utils.TimeTransferString(utils.FormatDateTime, origin.ModifyTime)
	return
}

// MultiUpdateBaseFromFenweiDataValue 批量更新汾渭指标数据
func MultiUpdateBaseFromFenweiDataValue(items []*BaseFromFenweiData) (err error) {
	if len(items) == 0 {
		return
	}

	//o := orm.NewOrm()
	to := global.DEFAULT_DB.Begin()

	defer func() {
		if err != nil {
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()
	sql := `UPDATE base_from_fenwei_data SET value = ?, modify_time = NOW() WHERE index_code = ? AND data_time = ? LIMIT 1`
	for _, v := range items {
		if v.IndexCode == "" || v.DataTime == "" {
			continue
		}
		err = to.Exec(sql, v.Value, v.IndexCode, v.DataTime).Error
		if err != nil {
			return
		}
	}

	return
}

// GetBaseFromFenWeiIndexByIndexName 根据指标名称查询指标
func GetBaseFromFenWeiIndexByIndexName(indexName string) (item *BaseFromFenweiIndex, err error) {
	//o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_fenwei_index WHERE index_name=? `
	//err = o.Raw(sql, indexName).QueryRow(&item)
	err = global.DEFAULT_DB.Raw(sql, indexName).First(&item).Error
	if utils.IsErrNoRow(err) {
		return nil, nil
	}
	return
}

// GetBaseFromFenweiDataByIndexCodeAndDataTime 根据指标编码和dataTime查询指标数据
func GetBaseFromFenweiDataByIndexCodeAndDataTime(indexCode, dataTime string) (item *BaseFromFenweiData, err error) {
	//o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_fenwei_data WHERE index_code=? AND data_time=? `
	//err = o.Raw(sql, indexCode, dataTime).QueryRow(&item)
	err = global.DEFAULT_DB.Raw(sql, indexCode, dataTime).First(&item).Error
	if utils.IsErrNoRow(err) {
		return nil, nil
	}

	return
}

// BatchAddBaseFromFenWeiData 批量插入指标数据
func BatchAddBaseFromFenWeiData(items []*BaseFromFenweiData) (err error) {
	if len(items) == 0 {
		return
	}

	//o := orm.NewOrm()
	//_, err = o.InsertMulti(1, items)
	err = global.DEFAULT_DB.CreateInBatches(items, 1).Error
	return
}

// AddBaseFromFenWeiClassify 新增汾渭分类
func AddBaseFromFenWeiClassify(item *BaseFromFenweiClassify) (classifyId int64, err error) {
	//o := orm.NewOrm()
	//classifyId, err = o.Insert(item)
	err = global.DEFAULT_DB.Create(&item).Error
	if err != nil {
		return
	}
	classifyId = int64(item.ClassifyId)
	return
}

// GetBaseFromFenWeiIndexByIndexCode 跟据指标编码查询指标信息
func GetBaseFromFenWeiIndexByIndexCode(indexCode string) (item *BaseFromFenweiIndex, err error) {
	//o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_fenwei_index WHERE index_code=? `
	//err = o.Raw(sql, indexCode).QueryRow(&item)
	err = global.DEFAULT_DB.Raw(sql, indexCode).First(&item).Error
	if utils.IsErrNoRow(err) {
		return nil, nil
	}

	return
}

// UpdateBaseFromFenweiData 更新汾渭数据
func UpdateBaseFromFenweiData(data *BaseFromFenweiData) error {
	//o := orm.NewOrm()
	//_, err := o.Update(data)
	err := global.DEFAULT_DB.Save(&data).Error
	return err
}
