package models

import (
	"eta/eta_index_lib/global"
	"eta/eta_index_lib/utils"
	"fmt"
	"strings"
	"time"
)

// CCFStockExcel CCF化纤装置表格
type CCFStockExcel struct {
	CcfStockExcelId int       `gorm:"column:ccf_stock_excel_id;type:int(10) UNSIGNED;primaryKey;not null;"`
	ClassifyId      int       `gorm:"column:classify_id;type:int(10) UNSIGNED;comment:分类ID;not null;default:0;"` // 分类ID
	ExcelDate       time.Time `gorm:"column:excel_date;type:date;comment:表格日期;default:NULL;"`                    // 表格日期
	ExcelContent    string    `gorm:"column:excel_content;type:longtext;comment:表格html信息;"`                      // 表格html信息
	FromPage        string    `gorm:"column:from_page;type:varchar(255);comment:表格来源;not null;"`                 // 表格来源
	CreateTime      time.Time `gorm:"column:create_time;type:datetime;comment:创建时间;default:NULL;"`               // 创建时间
	ModifyTime      time.Time `gorm:"column:modify_time;type:datetime;comment:更新时间;default:NULL;"`               // 更新时间
}

func (m *CCFStockExcel) TableName() string {
	return "ccf_stock_excel"
}

type CCFStockExcelCols struct {
	CcfStockExcelId string
	ClassifyId      string
	ExcelDate       string
	ExcelContent    string
	FromPage        string
	CreateTime      string
	ModifyTime      string
}

func (m *CCFStockExcel) Cols() CCFStockExcelCols {
	return CCFStockExcelCols{
		CcfStockExcelId: "ccf_stock_excel_id",
		ClassifyId:      "classify_id",
		ExcelDate:       "excel_date",
		ExcelContent:    "excel_content",
		FromPage:        "from_page",
		CreateTime:      "create_time",
		ModifyTime:      "modify_time",
	}
}

func (m *CCFStockExcel) PrimaryId() string {
	return m.Cols().CcfStockExcelId
}

func (m *CCFStockExcel) Create() (err error) {
	err = global.DEFAULT_DB.Create(m).Error

	return
}

func (m *CCFStockExcel) CreateMulti(items []*CCFStockExcel) (err error) {
	if len(items) == 0 {
		return
	}
	err = global.DEFAULT_DB.CreateInBatches(items, utils.MultiAddNum).Error

	return
}

func (m *CCFStockExcel) Update(cols []string) (err error) {
	err = global.DEFAULT_DB.Model(m).Select(cols).Updates(m).Error

	return
}

func (m *CCFStockExcel) Del() (err error) {
	sql := fmt.Sprintf(`DELETE FROM %s WHERE %s = ? LIMIT 1`, m.TableName(), m.PrimaryId())
	err = global.DEFAULT_DB.Exec(sql, m.CcfStockExcelId).Error

	return
}

func (m *CCFStockExcel) MultiDel(menuIds []int) (err error) {
	if len(menuIds) == 0 {
		return
	}
	sql := fmt.Sprintf(`DELETE FROM %s WHERE %s IN (%s)`, m.TableName(), m.PrimaryId(), utils.GetOrmInReplace(len(menuIds)))
	err = global.DEFAULT_DB.Exec(sql, menuIds).Error

	return
}

func (m *CCFStockExcel) GetItemById(id int) (item *CCFStockExcel, err error) {
	sql := fmt.Sprintf(`SELECT * FROM %s WHERE %s = ? LIMIT 1`, m.TableName(), m.PrimaryId())
	err = global.DEFAULT_DB.Raw(sql, id).First(&item).Error

	return
}

func (m *CCFStockExcel) GetItemByCondition(condition string, pars []interface{}, orderRule string) (item *CCFStockExcel, err error) {
	order := ``
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT * FROM %s WHERE 1=1 %s %s LIMIT 1`, m.TableName(), condition, order)
	err = global.DEFAULT_DB.Raw(sql, pars...).First(&item).Error

	return
}

func (m *CCFStockExcel) GetCountByCondition(condition string, pars []interface{}) (count int, err error) {
	sql := fmt.Sprintf(`SELECT COUNT(1) FROM %s WHERE 1=1 %s`, m.TableName(), condition)
	err = global.DEFAULT_DB.Raw(sql, pars...).Scan(&count).Error

	return
}

func (m *CCFStockExcel) GetItemsByCondition(condition string, pars []interface{}, fieldArr []string, orderRule string) (items []*CCFStockExcel, err error) {
	fields := strings.Join(fieldArr, ",")
	if len(fieldArr) == 0 {
		fields = `*`
	}
	order := fmt.Sprintf(`ORDER BY %s DESC`, m.Cols().CreateTime)
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT %s FROM %s WHERE 1=1 %s %s`, fields, m.TableName(), condition, order)
	err = global.DEFAULT_DB.Raw(sql, pars...).Find(&items).Error

	return
}

func (m *CCFStockExcel) GetPageItemsByCondition(condition string, pars []interface{}, fieldArr []string, orderRule string, startSize, pageSize int) (items []*CCFStockExcel, err error) {
	fields := strings.Join(fieldArr, ",")
	if len(fieldArr) == 0 {
		fields = `*`
	}
	order := fmt.Sprintf(`ORDER BY %s DESC`, m.Cols().CreateTime)
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT %s FROM %s WHERE 1=1 %s %s LIMIT ?,?`, fields, m.TableName(), condition, order)
	pars = append(pars, startSize, pageSize)
	err = global.DEFAULT_DB.Raw(sql, pars...).Find(&items).Error

	return
}
