package models

import (
	"eta/eta_index_lib/global"
	"eta/eta_index_lib/utils"
	"fmt"
	"strings"
	"time"
)

// FactorEdbSeriesCalculateData 因子指标系列-指标计算数据表
type FactorEdbSeriesCalculateData struct {
	FactorEdbSeriesCalculateDataId int       `gorm:"primaryKey;autoIncrement;column:factor_edb_series_calculate_data_id" description:"因子指标系列计算数据ID"`
	FactorEdbSeriesId              int       `gorm:"column:factor_edb_series_id" description:"因子指标系列ID"`
	EdbInfoId                      int       `gorm:"column:edb_info_id" description:"指标ID"`
	EdbCode                        string    `gorm:"column:edb_code" description:"指标编码"`
	DataTime                       time.Time `gorm:"column:data_time" description:"数据日期"`
	Value                          float64   `gorm:"column:value" description:"数据值"`
	CreateTime                     time.Time `gorm:"column:create_time" description:"创建时间"`
	ModifyTime                     time.Time `gorm:"column:modify_time" description:"修改时间"`
	DataTimestamp                  int64     `gorm:"column:data_timestamp" description:"数据日期时间戳"`
}

func (m *FactorEdbSeriesCalculateData) TableName() string {
	return "factor_edb_series_calculate_data"
}

type FactorEdbSeriesCalculateDataCols struct {
	PrimaryId         string
	FactorEdbSeriesId string
	EdbInfoId         string
	EdbCode           string
	DataTime          string
	Value             string
	CreateTime        string
	ModifyTime        string
	DataTimestamp     string
}

func (m *FactorEdbSeriesCalculateData) Cols() FactorEdbSeriesCalculateDataCols {
	return FactorEdbSeriesCalculateDataCols{
		PrimaryId:         "factor_edb_series_calculate_data_id",
		FactorEdbSeriesId: "factor_edb_series_id",
		EdbInfoId:         "edb_info_id",
		EdbCode:           "edb_code",
		DataTime:          "data_time",
		Value:             "value",
		CreateTime:        "create_time",
		ModifyTime:        "modify_time",
		DataTimestamp:     "data_timestamp",
	}
}

func (m *FactorEdbSeriesCalculateData) Create() (err error) {

	err = global.DEFAULT_DB.Create(m).Error
	if err != nil {
		return
	}
	return
}

func (m *FactorEdbSeriesCalculateData) CreateMulti(items []*FactorEdbSeriesCalculateData) (err error) {
	if len(items) == 0 {
		return
	}
	err = global.DEFAULT_DB.CreateInBatches(items, utils.MultiAddNum).Error

	return
}

func (m *FactorEdbSeriesCalculateData) Update(cols []string) (err error) {
	err = global.DEFAULT_DB.Model(m).Select(cols).Updates(m).Error

	return
}

func (m *FactorEdbSeriesCalculateData) Remove() (err error) {
	sql := fmt.Sprintf(`DELETE FROM %s WHERE %s = ? LIMIT 1`, m.TableName(), m.Cols().PrimaryId)
	err = global.DEFAULT_DB.Exec(sql, m.FactorEdbSeriesCalculateDataId).Error

	return
}

func (m *FactorEdbSeriesCalculateData) MultiRemove(ids []int) (err error) {
	if len(ids) == 0 {
		return
	}
	sql := fmt.Sprintf(`DELETE FROM %s WHERE %s IN (%s)`, m.TableName(), m.Cols().PrimaryId, utils.GetOrmInReplace(len(ids)))
	err = global.DEFAULT_DB.Exec(sql, ids).Error

	return
}

func (m *FactorEdbSeriesCalculateData) RemoveByCondition(condition string, pars []interface{}) (err error) {
	if condition == "" {
		return
	}

	sql := fmt.Sprintf(`DELETE FROM %s WHERE %s`, m.TableName(), condition)
	err = global.DEFAULT_DB.Exec(sql, pars...).Error

	return
}

func (m *FactorEdbSeriesCalculateData) GetItemById(id int) (item *FactorEdbSeriesCalculateData, err error) {
	sql := fmt.Sprintf(`SELECT * FROM %s WHERE %s = ? LIMIT 1`, m.TableName(), m.Cols().PrimaryId)
	err = global.DEFAULT_DB.Raw(sql, id).First(&item).Error

	return
}

func (m *FactorEdbSeriesCalculateData) GetItemByCondition(condition string, pars []interface{}, orderRule string) (item *FactorEdbSeriesCalculateData, err error) {
	order := ``
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT * FROM %s WHERE 1=1 %s %s LIMIT 1`, m.TableName(), condition, order)
	err = global.DEFAULT_DB.Raw(sql, pars...).First(&item).Error

	return
}

func (m *FactorEdbSeriesCalculateData) GetCountByCondition(condition string, pars []interface{}) (count int, err error) {
	sql := fmt.Sprintf(`SELECT COUNT(1) FROM %s WHERE 1=1 %s`, m.TableName(), condition)
	err = global.DEFAULT_DB.Raw(sql, pars...).Scan(&count).Error

	return
}

func (m *FactorEdbSeriesCalculateData) GetItemsByCondition(condition string, pars []interface{}, fieldArr []string, orderRule string) (items []*FactorEdbSeriesCalculateData, err error) {
	fields := strings.Join(fieldArr, ",")
	if len(fieldArr) == 0 {
		fields = `*`
	}
	order := fmt.Sprintf(`ORDER BY %s DESC`, m.Cols().CreateTime)
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT %s FROM %s WHERE 1=1 %s %s`, fields, m.TableName(), condition, order)
	err = global.DEFAULT_DB.Raw(sql, pars...).Find(&items).Error

	return
}

func (m *FactorEdbSeriesCalculateData) GetPageItemsByCondition(condition string, pars []interface{}, fieldArr []string, orderRule string, startSize, pageSize int) (items []*FactorEdbSeriesCalculateData, err error) {
	fields := strings.Join(fieldArr, ",")
	if len(fieldArr) == 0 {
		fields = `*`
	}
	order := fmt.Sprintf(`ORDER BY %s DESC`, m.Cols().CreateTime)
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT %s FROM %s WHERE 1=1 %s %s LIMIT ?,?`, fields, m.TableName(), condition, order)
	pars = append(pars, startSize, pageSize)
	err = global.DEFAULT_DB.Raw(sql, pars...).Find(&items).Error

	return
}

// FactorEdbSeriesCalculateDataItem 因子指标系列-计算数据信息
type FactorEdbSeriesCalculateDataItem struct {
	DataId            int     `description:"数据ID"`
	FactorEdbSeriesId int     `description:"因子指标系列ID"`
	EdbInfoId         int     `description:"指标ID"`
	EdbCode           string  `description:"指标编码"`
	DataTime          string  `description:"数据日期"`
	Value             float64 `description:"数据值"`
}

func (m *FactorEdbSeriesCalculateData) Format2Item() (item *FactorEdbSeriesCalculateDataItem) {
	item = new(FactorEdbSeriesCalculateDataItem)
	item.DataId = m.FactorEdbSeriesCalculateDataId
	item.FactorEdbSeriesId = m.FactorEdbSeriesId
	item.EdbInfoId = m.EdbInfoId
	item.EdbCode = m.EdbCode

	return
}

// TransEdbSeriesCalculateData2EdbDataList 转换数据格式
func TransEdbSeriesCalculateData2EdbDataList(items []*FactorEdbSeriesCalculateData) (list []*EdbInfoSearchData) {
	list = make([]*EdbInfoSearchData, 0)
	for _, v := range items {
		list = append(list, &EdbInfoSearchData{
			DataTime: v.DataTime.Format(utils.FormatDate),
			Value:    v.Value,
		})
	}

	return
}
