package models

import (
	"eta/eta_index_lib/global"
	"eta/eta_index_lib/utils"
	"fmt"
	"strings"
	"time"
)

// FactorEdbSeriesCalculateFunc 多因子系列-计算方式表
type FactorEdbSeriesCalculateFunc struct {
	FactorEdbSeriesCalculateFuncId int       `gorm:"primaryKey;autoIncrement;column:factor_edb_series_calculate_func_id" description:"多因子系列计算方式ID"`
	CalculateName                  string    `gorm:"column:calculate_name" description:"计算方式名称"`
	Source                         int       `gorm:"column:source" description:"计算方式来源"`
	EdbInfoType                    int       `gorm:"column:edb_info_type" description:"指标计算类型：0-普通指标；1-预测指标"`
	CreateTime                     time.Time `gorm:"column:create_time" description:"创建时间"`
	ModifyTime                     time.Time `gorm:"column:modify_time" description:"修改时间"`
}

func (m *FactorEdbSeriesCalculateFunc) TableName() string {
	return "factor_edb_series_calculate_func"
}

type FactorEdbSeriesCalculateFuncCols struct {
	PrimaryId     string
	CalculateName string
	Source        string
	EdbInfoType   string
	CreateTime    string
	ModifyTime    string
}

func (m *FactorEdbSeriesCalculateFunc) Cols() FactorEdbSeriesCalculateFuncCols {
	return FactorEdbSeriesCalculateFuncCols{
		PrimaryId:     "factor_edb_series_calculate_func_id",
		CalculateName: "calculate_name",
		Source:        "source",
		EdbInfoType:   "edb_info_type",
		CreateTime:    "create_time",
		ModifyTime:    "modify_time",
	}
}

func (m *FactorEdbSeriesCalculateFunc) Create() (err error) {
	err = global.DEFAULT_DB.Create(m).Error

	return
}

func (m *FactorEdbSeriesCalculateFunc) CreateMulti(items []*FactorEdbSeriesCalculateFunc) (err error) {
	if len(items) == 0 {
		return
	}

	err = global.DEFAULT_DB.CreateInBatches(items, utils.MultiAddNum).Error

	return
}

func (m *FactorEdbSeriesCalculateFunc) Update(cols []string) (err error) {
	err = global.DEFAULT_DB.Model(m).Select(cols).Updates(m).Error

	return
}

func (m *FactorEdbSeriesCalculateFunc) Remove() (err error) {
	sql := fmt.Sprintf(`DELETE FROM %s WHERE %s = ? LIMIT 1`, m.TableName(), m.Cols().PrimaryId)
	err = global.DEFAULT_DB.Exec(sql, m.FactorEdbSeriesCalculateFuncId).Error

	return
}

func (m *FactorEdbSeriesCalculateFunc) MultiRemove(ids []int) (err error) {
	if len(ids) == 0 {
		return
	}

	sql := fmt.Sprintf(`DELETE FROM %s WHERE %s IN (%s)`, m.TableName(), m.Cols().PrimaryId, utils.GetOrmInReplace(len(ids)))
	err = global.DEFAULT_DB.Exec(sql, ids).Error
	return
}

func (m *FactorEdbSeriesCalculateFunc) GetItemById(id int) (item *FactorEdbSeriesCalculateFunc, err error) {

	sql := fmt.Sprintf(`SELECT * FROM %s WHERE %s = ? LIMIT 1`, m.TableName(), m.Cols().PrimaryId)
	err = global.DEFAULT_DB.Raw(sql, id).First(&item).Error
	return
}

func (m *FactorEdbSeriesCalculateFunc) GetItemByCondition(condition string, pars []interface{}, orderRule string) (item *FactorEdbSeriesCalculateFunc, err error) {

	order := ``
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT * FROM %s WHERE 1=1 %s %s LIMIT 1`, m.TableName(), condition, order)
	err = global.DEFAULT_DB.Raw(sql, pars...).First(&item).Error
	return
}

func (m *FactorEdbSeriesCalculateFunc) GetCountByCondition(condition string, pars []interface{}) (count int, err error) {

	sql := fmt.Sprintf(`SELECT COUNT(1) FROM %s WHERE 1=1 %s`, m.TableName(), condition)
	err = global.DEFAULT_DB.Raw(sql, pars...).Scan(&count).Error
	return
}

func (m *FactorEdbSeriesCalculateFunc) GetItemsByCondition(condition string, pars []interface{}, fieldArr []string, orderRule string) (items []*FactorEdbSeriesCalculateFunc, err error) {

	fields := strings.Join(fieldArr, ",")
	if len(fieldArr) == 0 {
		fields = `*`
	}
	order := fmt.Sprintf(`ORDER BY %s DESC`, m.Cols().CreateTime)
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT %s FROM %s WHERE 1=1 %s %s`, fields, m.TableName(), condition, order)
	err = global.DEFAULT_DB.Raw(sql, pars...).Find(&items).Error
	return
}

func (m *FactorEdbSeriesCalculateFunc) GetPageItemsByCondition(condition string, pars []interface{}, fieldArr []string, orderRule string, startSize, pageSize int) (items []*FactorEdbSeriesCalculateFunc, err error) {

	fields := strings.Join(fieldArr, ",")
	if len(fieldArr) == 0 {
		fields = `*`
	}
	order := fmt.Sprintf(`ORDER BY %s DESC`, m.Cols().CreateTime)
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT %s FROM %s WHERE 1=1 %s %s LIMIT ?,?`, fields, m.TableName(), condition, order)
	pars = append(pars, startSize, pageSize)
	err = global.DEFAULT_DB.Raw(sql, pars...).Find(&items).Error
	return
}

// FactorEdbSeriesCalculateFuncItem 多因子系列-计算方式
type FactorEdbSeriesCalculateFuncItem struct {
	CalculateName string `description:"计算方式名称"`
	Source        int    `description:"计算方式来源"`
}

func (m *FactorEdbSeriesCalculateFunc) Format2Item() (item *FactorEdbSeriesCalculateFuncItem) {
	item = new(FactorEdbSeriesCalculateFuncItem)
	item.CalculateName = m.CalculateName
	item.Source = m.Source
	return
}
