package future_good

import (
	"eta/eta_index_lib/global"
	"fmt"
	"strings"
	"time"
)

type ChartInfoFutureGoodProfit struct {
	ChartInfoId  int       `gorm:"column:chart_info_id;type:int(9);primaryKey;not null;"`
	ProfitName   string    `gorm:"column:profit_name;type:varchar(255);comment:利润名称;"`                           // 利润名称
	ProfitNameEn string    `gorm:"column:profit_name_en;type:varchar(255);comment:利润英文名称;"`                      // 利润英文名称
	XValue       string    `gorm:"column:x_value;type:text;comment:x轴的数据值;"`                                     // x轴的数据值
	YValue       string    `gorm:"column:y_value;type:text;comment:y轴的数据值;"`                                     // y轴的数据值
	ModifyTime   time.Time `gorm:"column:modify_time;type:timestamp;comment:最近的修改时间;default:CURRENT_TIMESTAMP;"` // 最近的修改时间
	CreateTime   time.Time `gorm:"column:create_time;type:datetime;comment:创建时间;default:NULL;"`                  // 创建时间
}

type CorrelationInfo struct {
	LeadValue       int    `description:"领先值"`
	LeadUnit        string `description:"领先单位"`
	CalculateValue  int    `description:"计算窗口"`
	CalculateUnit   string `description:"计算频度"`
	StartDate       string `description:"开始日期"`
	EndDate         string `description:"结束日期"`
	EdbInfoIdFirst  int    `description:"A指标ID"`
	EdbInfoIdSecond int    `description:"B指标ID"`
	PeriodData      string `description:"X轴-期数数据"`
	CorrelationData string `description:"Y轴-商品利润系数"`
}

func (m *ChartInfoFutureGoodProfit) TableName() string {
	return "chart_info_future_good_profit"
}

func (m *ChartInfoFutureGoodProfit) Create() (err error) {
	err = global.DEFAULT_DB.Create(m).Error

	return
}

func (m *ChartInfoFutureGoodProfit) Update(cols []string) (err error) {
	err = global.DEFAULT_DB.Model(m).Select(cols).Updates(m).Error

	return
}

func (m *ChartInfoFutureGoodProfit) Delete() (err error) {
	sql := fmt.Sprintf(`DELETE FROM %s WHERE chart_info_id = ? LIMIT 1`, m.TableName())
	err = global.DEFAULT_DB.Exec(sql, m.ChartInfoId).Error

	return
}

func (m *ChartInfoFutureGoodProfit) GetItemById(id int) (err error) {
	sql := fmt.Sprintf(`SELECT * FROM %s WHERE chart_info_id = ? LIMIT 1`, m.TableName())
	err = global.DEFAULT_DB.Raw(sql, id).First(&m).Error

	return
}

func (m *ChartInfoFutureGoodProfit) GetItemByCondition(condition string, pars []interface{}) (err error) {
	sql := fmt.Sprintf(`SELECT * FROM %s WHERE 1=1 %s LIMIT 1`, m.TableName(), condition)
	err = global.DEFAULT_DB.Raw(sql, pars...).First(&m).Error

	return
}

func (m *ChartInfoFutureGoodProfit) GetItemsByCondition(condition string, pars []interface{}, fieldArr []string, orderRule string) (items []*ChartInfoFutureGoodProfit, err error) {
	fields := strings.Join(fieldArr, ",")
	if len(fieldArr) == 0 {
		fields = `*`
	}
	order := ``
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT %s FROM %s WHERE 1=1 %s %s`, fields, m.TableName(), condition, order)
	err = global.DEFAULT_DB.Raw(sql, pars...).Find(&items).Error

	return
}
