package trade_analysis

// CorrelationTableExtraConfig 相关性表格配置
type CorrelationTableExtraConfig struct {
	BaseEdbInfoId    int                          `description:"标的指标ID"`
	Exchange         string                       `description:"交易所标识"`
	ClassifyName     string                       `description:"(单选)品种"`
	CompanyNames     []string                     `description:"(多选)期货公司"`
	ContractType     int                          `description:"合约类型: 1-主力合约; 2-成交量前2; 3-成交量前3; 4-所有合约(多个); 5-合约加总(1个)"`
	ContractPosition int                          `description:"(单选)合约方向: 1-多； 2-空; 3-净多"`
	PredictRatio     float64                      `description:"预估参数, 0-1之间"`
	RollConfig       []CorrelationTableRollConfig `description:"滚动相关性配置"`
}

// CorrelationTableRollConfig 滚动相关性配置
type CorrelationTableRollConfig struct {
	CalculateValue int `description:"计算窗口"`
	LeadValue      int `description:"领先期数"`
}

// CorrelationTableRowData 相关性表格行数据
type CorrelationTableRowData struct {
	Exchange     string                        `description:"交易所"`
	ClassifyName string                        `description:"品种"`
	ClassifyType string                        `description:"合约"`
	CompanyName  string                        `description:"公司名称"`
	RowName      string                        `description:"合约持仓名称"`
	DayData      []*CorrelationTableRowDayData `description:"天数对应的相关性系数"`
}

// CorrelationTableRowDayData 相关性表格行数据值
type CorrelationTableRowDayData struct {
	Day      int     `description:"天数: 从0到-10, 0为最新数据, -1表示前一天的滚动相关性"`
	DataDate string  `description:"对应的日期"`
	DataVal  float64 `description:"相关性系数"`
}

// CorrelationTableData 相关性表格数据
type CorrelationTableData struct {
	CorrelationTableRollConfig `description:"滚动相关性配置, 每个配置单独为一张表"`
	RowsData                   []*CorrelationTableRowData `description:"表格行数据"`
}
